
/* Comments formatted for doc++ */

/**@name War Ftp daemon 1.70 DLL API Interface.

   Include the file {\tt"War170API.h"}.
   This file is compatible with ANSI C and ANSI C++ 


   Link with {\tt"War170API.lib"}, or load the DLL 
   functions manually.

   Redistribution of the {\tt"War170API.dll"} is unlimited.

   {\bfDisclaimer: Nor the author or anyone else is responsible for 
   any damage, loss of income, warfare, intergalactic crises
   or any other minor or major problems or disasters caused 
   by this software.}

   @author Jarle (jgaa) Aase. Copyright 2000 Jarle (jgaa) Aase. All rights reserved.
   @version 1.0 Compatible with War FTP Daemon 1.71
  */
/*@{*/

#ifndef WAR170API_H
#define WAR170API_H

#define WAR_API_INTERFACE_VERISON 1

#ifdef WAR170API_EXPORTS
#define WAR170API_API __declspec(dllexport)
#else
#define WAR170API_API __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define WAR_IN
#define WAR_OUT

/**@name Error codes */
/*@{*/
/** Value 0 */
#define WARAPI_ERR_OK 0
/** Value 1  */
#define WARAPI_ERR_UNDEFINED 1
/** Value 2 */
#define WARAPI_ERR_UNSUPPORTED 2
/** Value 3 */
#define WARAPI_ERR_EINVAL 3
/** Value 4 */
#define WARAPI_ERR_COMMFAIL 4
/** Value 5 */
#define WARAPI_ERR_TIMEOUT 5
/** Value 6. Authentication failed. Bad username or password */
#define WARAPI_ERR_AUTH 6 
/** Value 7. Unsificcient permissions to complete request*/
#define WARAPI_ERR_PERM 7
/** Value 8. An object was not found */
#define WARAPI_ERR_NOT_FOUND 8 
/** Value 9 */
#define WARAPI_ERR_UNDEFINED_SEARCHERR 9
/** Value 10 */
#define WARAPI_ERR_NOT_A_GROUP 10
/** Value 11 */
#define WARAPI_ERR_CREATE_FAILED_UNDEFINED 11
/** Value 12. A search was successful, but the mask did not match the node-type */
#define WARAPI_ERR_SRCMASK_MISMATCH 12
/** Value 13. The /system/class/group pat was syntaxly incorrect */
#define WARAPI_ERR_INVALID_GROUPPATH 13 
/** Value 14. The user already exist */
#define WARAPI_ERR_USER_EXIST 14 
/*@}*/

/**@name Return values from API calls */
/*@{*/
#define WARAPI_CALL_OK 0
/**  */
#define WARAPI_CALL_GENERIC_FAIL 1
/**  */
#define WARAPI_CALL_EINVAL 2
/*@}*/

#define WARAPI_DEFAULT_LOGIN "Sysadmin"
#define WARAPI_DEFAULT_PORT 0

/**@name API Datatypes

    IMPORTANT: The datatypes may change in the future. 
    Always use the typedef'ed names!
*/
/*@{*/
/**  */
typedef void * WARAPI_USER_HANDLE;
/**  */
typedef void * WARAPI_SESSION_HANDLE;
/**  */
typedef const char * WARAPI_CCSTR_T;
/**  */
typedef const int WARAPI_RESULT;
/**  */
typedef unsigned short WARAPI_PORT;
/**  */
typedef unsigned int WARAPI_SEARCH_MASK;
/** */
typedef unsigned int WARAPI_FLAGS;
/** */
typedef unsigned int WARAPI_BOOL_T;
/**@name WARAPI_LOGIN_TYPE

   Enumerated login permission types

   @see Warapi_SetLoginPermission
*/
/*@{*/
typedef enum WARAPI_LOGIN_TYPE_E
{
    /** Login perm to the FTP server */
    WARAPI_LOGIN_TYPE_FTPUSER = 0,
    /** Login perm to the administrator console */
    WARAPI_LOGIN_TYPE_ADMIN = 1
} WARAPI_LOGIN_TYPE;
/*@}*/
/**@name WARAPI_LOGIN_PERM

   Enumerated login permission types

   @see Warapi_SetLoginPermission
*/
/*@{*/
typedef enum WARAPI_LOGIN_PERM_E
{
    /** Deny access */
    WARAPI_LOGIN_PERM_DENY = 0,
    /** Allow access */
    WARAPI_LOGIN_PERM_ALLOW = 1,
    /** Use default from lower level */
    WARAPI_LOGIN_PERM_DEFAULT = 2
} WARAPI_LOGIN_PERM;
/*@}*/
/*@}*/

/**@name Search options
   
   @see Warapi_LookupUser
*/
/*@{*/
/**Return virtual domain nodes */
#define WARAPI_SEARCH_MASK_DOMAIN   0x00000001
/**Return class nodes */
#define WARAPI_SEARCH_MASK_CLASS    0x00000002
/**return group nodes  */
#define WARAPI_SEARCH_MASK_GROUP    0x00000004
/**return user nodes  */
#define WARAPI_SEARCH_MASK_USER     0x00000008
/**return alias nodes  */
#define WARAPI_SEARCH_MASK_ALIAS    0x00000010
/**return alll types */
#define WARAPI_SEARCH_MASK_ALL      0x0000001f
/*@}*/

/**@name Permission flags */
/*@{*/
/** read files */
#define WARAPI_PERM_READ            0x00000001
/** write to and create files */
#define WARAPI_PERM_WRITE           0x00000002
/** Change to directory */
#define WARAPI_PERM_EXECUTE         0x00000004
/** List content of directory */
#define WARAPI_PERM_LIST            0x00000008
/** delete files */
#define WARAPI_PERM_DELETE          0x00000010
/** Remove directories */
#define WARAPI_PERM_REMOVE          0x00000020
/** Create directories */
#define WARAPI_PERM_CREATE          0x00000040
/** Hide the directory */
#define WARAPI_PERM_HIDE            0x00000080
/** The permissiosn are recursive, that is; applies for subdirectories as well */
#define WARAPI_PERM_RECURSIVE       0x00000100
/** Deny all access */
#define WARAPI_PERM_DENY            0x00000200
/** Free files. Ratios are disabled */
#define WARAPI_PERM_FREE            0x00000400
/*@}*/

/**@name API Interface */
/*@{*/

/** Initializes the War API DLL.

    This function must be called for each thread that use
    the War FTP Daemon API library, prior to using any
    other functions in the library.

    @param Version WAR_API_INTERFACE_VERISON

    @param ErrorValue A error value to indicate why the operation failed.
        This value will be WARAPI_ERR_OK if the function succeeded.
        Possible error values are:
        \begin{itemize}
            \item WARAPI_ERR_UNSUPPORTED The loaded DLL is not 
                compatible with the API header file you are using.
        \end{itemize}

    @return WARAPI_CALL_OK if the API was initialized OK. 
*/
WAR170API_API WARAPI_RESULT 
Warapi_InitializeApi(const WAR_OUT int Version, 
                     WAR_IN int *ErrorValue);


/** Open a session to War FTP Daemon.

    This function initiates a RPC communication session with
    War FTP Daemon.

    @param Host Hostname. This can be a IP address like "192.168.1.100",
        an Internet domain name like "ftp.jgaa.com" or simply "localhost"
        if you are connecting to the local machine.

    @param Port The port number used by teh server. If the server use
        the portmapper, use the WARAPI_DEFAULT_PORT macro

    @param LoginName The login name to log on as. This user account must 
        have remote access privilegies. You can use the WARAPI_DEFAULT_LOGIN
        macro to specify the default system administrator account.

    @param Password The password for the login account.

    @param SessionHandle Pointer to a WARAPI_SESSION_HANDLE variable
        receive the login handle if the funcion succeed.

    @param ErrorValue A error value to indicate why the operation failed.
        This value will be WARAPI_ERR_OK if the function succeeded.

    @return WARAPI_CALL_OK if the function succeeded.

    @see Warapi_CloseSession
*/
WAR170API_API WARAPI_RESULT 
Warapi_OpenSession(const WAR_OUT WARAPI_CCSTR_T Host, 
                   const WAR_OUT WARAPI_PORT Port,
                   const WAR_OUT WARAPI_CCSTR_T LoginName,
                   const WAR_OUT WARAPI_CCSTR_T Password,
                   WAR_IN WARAPI_SESSION_HANDLE *SessionHandle,
                   WAR_IN int *ErrorValue);

/** Find an existing node in the user database.

    Note: The '*' character should not be used in the name,
    as this is used to search for patterns in some versions
    of the server. The use of pattern-matching characters 
    are undefined, and may lead to unexpected and unpredictible 
    results.

    @param SessionHandle Previously initialized session handle
    
    @param Name Path to the node, like {\tt"/system/user/johndoe"}

    @param SearchMask Search parameters. See the WARAPI_SEARCH_MASK_* 
        macros. The macros can be ored together.
        
    @param UserHandle Pointer to a variable where an initialized user handle
        will be returned if the function succeeds. This handle must 
        be released with \Ref{Warapi_ReleaseUserHandle}.
  
    @param ErrorValue A error value to indicate why the operation failed.
        This value will be WARAPI_ERR_OK if the function succeeded.

    @return WARAPI_CALL_OK if the function succeeded.

    @see Warapi_ReleaseUserHandle Warapi_OpenSession
*/


WAR170API_API WARAPI_RESULT 
Warapi_LookupUser(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  const WAR_OUT WARAPI_CCSTR_T Name,
                  const WAR_OUT WARAPI_SEARCH_MASK SearchMask,
                  WAR_IN WARAPI_USER_HANDLE *UserHandle,
                  WAR_IN int *ErrorValue);

/** Create a user in the user database.

    Creates a user in the user tree. After the user is created,
    you will normally set individual properties for the user.

    @param SessionHandle Previously initialized session handle
    
    @param Parent Path to the parent node, like {\tt"/system/user"}
    
    @param Name Name of the new user
    
    @param UserHandle Pointer to a variable where an initialized user handle
        will be returned if the function succeeds. This handle must 
        be released with \Ref{Warapi_ReleaseUserHandle}.
  
    @param ErrorValue A error value to indicate why the operation failed.
        This value will be WARAPI_ERR_OK if the function succeeded.

    @return WARAPI_CALL_OK if the function succeeded.

    @see Warapi_ReleaseUserHandle Warapi_OpenSession

*/
WAR170API_API WARAPI_RESULT 
Warapi_CreateUser(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  const WAR_OUT WARAPI_CCSTR_T Parent, 
                  const WAR_OUT WARAPI_CCSTR_T Name,
                  WAR_IN WARAPI_USER_HANDLE *UserHandle,
                  WAR_IN int *ErrorValue);

/** Set the password for a user

    @param SessionHandle Previously initialized session handle

    @param UserHandle A previously initialized user handle

    @param Password The new password to use

    @param ErrorValue A error value to indicate why the operation failed.
        This value will be WARAPI_ERR_OK if the function succeeded.

    @return WARAPI_CALL_OK if the function succeeded.
*/
WAR170API_API WARAPI_RESULT 
Warapi_SetPassword(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  WAR_OUT WARAPI_USER_HANDLE UserHandle,
                  const WAR_OUT WARAPI_CCSTR_T Password,
                  WAR_IN int *ErrorValue);


/** Define or re-define a path for a user. 

    A path on the user-level overrides a similar path
    on a "lower" level.

    @param SessionHandle Previously initialized session handle

    @param UserHandle A previously initialized user handle

    @param VirtualPath The name/alias of the path. Typically
        {\tt"/pub"}. {\bfUses slash and not backslash.}. If no
        virtual path is defined, this can be NULL or an empty
        string.

    @param PhysPath The physical path. The path must exist on the
        local machine. UNC paths are allowed, but will only work
        when the server runs in non-servive mode. Example:
        {\tt"C:\\files\\documents\\ftp\\pub"}. Note that this
        argument use backslash.

    @param Flags The permissions to set. The flags below can be
        ored together. See \Ref{WARAPI_PERM_READ} and other
        \Ref{WARAPI_PERM_} flags.

    @param ErrorValue A error value to indicate why the operation failed.
        This value will be WARAPI_ERR_OK if the function succeeded.

    @return WARAPI_CALL_OK if the function succeeded.
*/
WAR170API_API WARAPI_RESULT 
Warapi_SetPath(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  WAR_OUT WARAPI_USER_HANDLE UserHandle,
                  const WAR_OUT WARAPI_CCSTR_T VirtualPath,
                  const WAR_OUT WARAPI_CCSTR_T PhysPath,
                  const WAR_OUT WARAPI_FLAGS Flags,
                  WAR_IN int *ErrorValue);

/** Set the login permission for a user

    @param SessionHandle Previously initialized session handle

    @param UserHandle A previously initialized user handle

    @param Type Type of permission to set. One of:
        \begin{itemize}
        \item {\ttWARAPI_LOGIN_TYPE_FTPUSER} Access to the FTP server as a normal user
        \item {\ttWARAPI_LOGIN_TYPE_ADMIN} Access to the remote access interface
        \end{itemize}

    @param Permission Permission to set. One of:
        \begin{itemize}
        \item {\ttWARAPI_LOGIN_PERM_DENY} Dey access. Overrides inherited permissions.
        \item {\ttWARAPI_LOGIN_PERM_ALLOW} Allow access. Overrides inherited permissions.
        \item {\ttWARAPI_LOGIN_PERM_DEFAULT} Use the default. This is recommended,
           as one can enable or disable access for all users in a group by setting
           the groups permission. 
        \end{itemize}

    @param ErrorValue A error value to indicate why the operation failed.
        This value will be WARAPI_ERR_OK if the function succeeded.

    @return WARAPI_CALL_OK if the function succeeded.
*/
WAR170API_API WARAPI_RESULT 
Warapi_SetLoginPermission(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  WAR_OUT WARAPI_USER_HANDLE UserHandle,
                  const WAR_OUT WARAPI_LOGIN_TYPE Type,
                  const WAR_OUT WARAPI_LOGIN_PERM Permission, 
                  WAR_IN int *ErrorValue);

/** Delete a user (or any other node-type) from the server.

    @param SessionHandle Previously initialized session handle
    
    @param Name Path to the node, like {\tt"/system/user/johndoe"}
  
    @param ErrorValue A error value to indicate why the operation failed.
        This value will be WARAPI_ERR_OK if the function succeeded.

    @return WARAPI_CALL_OK if the function succeeded.

    @see Warapi_OpenSession
*/
WAR170API_API WARAPI_RESULT 
Warapi_DeleteUser(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  const WAR_OUT WARAPI_CCSTR_T Name,
                  WAR_IN int *ErrorValue);

/** Release a user handle.

    Use this function to release a user handke.

    @param Handle Previously initialized handle
    @return WARAPI_CALL_OK if the function succeeded.
    @see Warapi_CreateUser Warapi_LookupUser
*/
WAR170API_API WARAPI_RESULT 
Warapi_ReleaseUserHandle(WARAPI_USER_HANDLE Handle);

/** Initiate a server shutdown

    @param SessionHandle Previously initialized session handle

    @param WaitForUsersToFinish 0 to shut down immediately, 1 to wait for
         the current transfers to complete.

    @param ErrorValue A error value to indicate why the operation failed.
        This value will be WARAPI_ERR_OK if the function succeeded.

    @return WARAPI_CALL_OK if the function succeeded.
*/
WAR170API_API WARAPI_RESULT 
Warapi_Shutdown(WAR_OUT WARAPI_SESSION_HANDLE SessionHandle,
                  const WARAPI_BOOL_T WaitForUsersToFinish,
                  WAR_IN int *ErrorValue);
  
/** Close a previously opened session with War FTP Daemon
  
    You must call this function to release system resources
    after you have completed a session, opened with Warapi_OpenSession

    @param SessionHandle A previously opened session handle

    @return WARAPI_CALL_OK if the function succeeded.

    @see Warapi_OpenSession 
*/
WAR170API_API WARAPI_RESULT 
Warapi_CloseSession(const WAR_OUT WARAPI_SESSION_HANDLE SessionHandle);

/*@}*/

#ifdef __cplusplus
} /* extern "C" */
#endif


#endif /* WAR170API_H */

/*@} API */ 

